# Generated by roxygen2: do not edit by hand

S3method(makeContractGridDimname,InsuranceTarif)
S3method(makeContractGridDimname,R6)
S3method(makeContractGridDimname,default)
S3method(makeContractGridDimname,double)
S3method(makeContractGridDimname,mortalityTable)
S3method(makeContractGridDimname,numeric)
export(CalculationEnum)
export(InsuranceContract)
export(InsuranceContract.ParameterDefaults)
export(InsuranceContract.ParameterStructure)
export(InsuranceContract.ParametersFallback)
export(InsuranceContract.ParametersFill)
export(InsuranceContract.Values)
export(InsuranceTarif)
export(PP.base.NULL)
export(PP.base.PreviousZillmerReserve)
export(PP.base.ZillmerReserveT2)
export(PP.base.ZillmerRiskPremium)
export(PP.base.contractualReserve)
export(PP.base.meanContractualReserve)
export(PP.base.previousContractualReserve)
export(PP.base.sumInsured)
export(PP.base.totalProfitAssignment)
export(PP.benefit.None)
export(PP.benefit.Profit)
export(PP.benefit.ProfitGuaranteeSupporting)
export(PP.benefit.ProfitPlusGuaranteedInterest)
export(PP.benefit.ProfitPlusHalfGuaranteedInterest)
export(PP.benefit.ProfitPlusHalfInterestMinGuaranteeTotal)
export(PP.benefit.ProfitPlusHalfTotalInterest)
export(PP.benefit.ProfitPlusInterestMinGuaranteeTotal)
export(PP.benefit.ProfitPlusTerminalBonusReserve)
export(PP.benefit.ProfitPlusTotalInterest)
export(PP.benefit.TerminalBonus)
export(PP.benefit.TerminalBonus5Years)
export(PP.benefit.TerminalBonus5YearsProRata)
export(PP.calculate.RateOnBase)
export(PP.calculate.RateOnBaseMin0)
export(PP.calculate.RateOnBaseSGFFactor)
export(PP.calculate.RatePlusGuaranteeOnBase)
export(PP.rate.expenseProfit)
export(PP.rate.interestProfit)
export(PP.rate.interestProfit2)
export(PP.rate.interestProfit2PlusGuarantee)
export(PP.rate.interestProfitPlusGuarantee)
export(PP.rate.riskProfit)
export(PP.rate.sumProfit)
export(PP.rate.terminalBonus)
export(PP.rate.terminalBonusFund)
export(PP.rate.totalInterest)
export(PP.rate.totalInterest2)
export(PVfactory)
export(PaymentTimeEnum)
export(ProfitComponentsEnum)
export(ProfitParticipation)
export(RoundingHelper)
export(SexEnum)
export(TariffTypeEnum)
export(age.exactRounded)
export(age.yearDifference)
export(applyHook)
export(contractGrid)
export(contractGridPremium)
export(costs.baseAlpha)
export(costs.scaleAlpha)
export(costsDisplayTable)
export(deathBenefit.annuityDecreasing)
export(deathBenefit.linearDecreasing)
export(exportInsuranceContract.xlsx)
export(exportInsuranceContractExample)
export(fallbackFields)
export(fillFields)
export(fillNAgaps)
export(filterProfitRates)
export(freqCharge)
export(head0)
export(initializeCosts)
export(isRegularPremiumContract)
export(isSinglePremiumContract)
export(makeContractGridDimname)
export(makeContractGridDimnames)
export(pad0)
export(padLast)
export(premiumRefundPeriod.default)
export(rollingmean)
export(setCost)
export(showVmGlgExamples)
export(sumProfits)
export(testVmGlgExample)
export(valueOrFunction)
export(vmGlgExample.generateTest)
exportClasses(CalculationSingleEnum)
exportClasses(PaymentTimeSingleEnum)
exportClasses(ProfitComponentsMultipleEnum)
exportClasses(SexSingleEnum)
exportClasses(TariffTypeSingleEnum)
import(MortalityTables)
import(R6)
import(dplyr)
import(lubridate)
import(openxlsx)
import(scales)
import(stringr)
import(tidyr)
importFrom(abind,abind)
importFrom(kableExtra,kable)
importFrom(lubridate,"year<-")
importFrom(lubridate,days)
importFrom(lubridate,interval)
importFrom(lubridate,month)
importFrom(lubridate,time_length)
importFrom(lubridate,year)
importFrom(lubridate,years)
importFrom(methods,is)
importFrom(methods,new)
importFrom(objectProperties,setSingleEnum)
importFrom(pander,pander)
importFrom(rlang,.data)
importFrom(rmarkdown,render)
importFrom(utils,head)
importFrom(utils,tail)
