# The *LifeInsureR* Package For Traditional Life Insurance with Guarantee and Profit Participation
R package implementing general life insurance contracts

This package R6 classes to model traditional life insurance
contracts like annuities, whole life insurances or endowments. All relevant
quantities like premium decomposition, reserves and benefits over the whole
contract period are calculated and potentially exported to 'Excel'. Mortalities
are given using the 'MortalityTables' package.
